@@ -5,7 +5,7 @@
 
     Several HTML builders.
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -161,8 +161,7 @@ class StandaloneHTMLBuilder(Builder):
             if docname not in self.env.all_docs:
                 yield docname
                 continue
-            targetname = self.env.doc2path(docname, self.outdir,
-                                           self.out_suffix)
+            targetname = self.get_outfilename(docname)
             try:
                 targetmtime = path.getmtime(targetname)
             except Exception:
@@ -416,6 +415,10 @@ class StandaloneHTMLBuilder(Builder):
                 else:
                     stripped = ''
 
+                # we stripped the whole module name
+                if not mn:
+                    continue
+
                 if fl != mn[0].lower() and mn[0] != '_':
                     # heading
                     letter = mn[0].upper()
@@ -642,7 +645,8 @@ class StandaloneHTMLBuilder(Builder):
                    baseuri=self.get_target_uri(pagename)):
             if not resource:
                 otheruri = self.get_target_uri(otheruri)
-            return relative_uri(baseuri, otheruri)
+            uri = relative_uri(baseuri, otheruri) or '#'
+            return uri
         ctx['pathto'] = pathto
         ctx['hasdoc'] = lambda name: name in self.env.all_docs
         ctx['customsidebar'] = self.config.html_sidebars.get(pagename)
@@ -652,7 +656,14 @@ class StandaloneHTMLBuilder(Builder):
         self.app.emit('html-page-context', pagename, templatename,
                       ctx, event_arg)
 
-        output = self.templates.render(templatename, ctx)
+        try:
+            output = self.templates.render(templatename, ctx)
+        except UnicodeError:
+            self.warn("a Unicode error occurred when rendering the page %s. "
+                      "Please make sure all config values that contain "
+                      "non-ASCII content are Unicode strings." % pagename)
+            return
+
         if not outfilename:
             outfilename = self.get_outfilename(pagename)
         # outfilename's path is in general different from self.outdir
