@@ -5,7 +5,7 @@
 
     Global creation environment.
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -21,13 +21,7 @@ import difflib
 import cPickle as pickle
 from os import path
 from glob import glob
-from string import ascii_uppercase as uppercase
 from itertools import izip, groupby
-try:
-    from hashlib import md5
-except ImportError:
-    # 2.4 compatibility
-    from md5 import md5
 
 from docutils import nodes
 from docutils.io import FileInput, NullOutput
@@ -43,7 +37,7 @@ from docutils.transforms.parts import ContentsFilter
 
 from sphinx import addnodes
 from sphinx.util import movefile, get_matching_docs, SEP, ustrftime, \
-     docname_join, FilenameUniqDict, url_re
+     docname_join, FilenameUniqDict, url_re, clean_astext
 from sphinx.errors import SphinxError
 from sphinx.directives import additional_xref_types
 
@@ -59,7 +53,7 @@ default_settings = {
 
 # This is increased every time an environment attribute is added
 # or changed to properly invalidate pickle files.
-ENV_VERSION = 30
+ENV_VERSION = 31
 
 
 default_substitutions = set([
@@ -826,11 +820,11 @@ class BuildEnvironment:
                           node.line)
             self.anonlabels[name] = docname, labelid
             if node.tagname == 'section':
-                sectname = node[0].astext() # node[0] == title node
+                sectname = clean_astext(node[0]) # node[0] == title node
             elif node.tagname == 'figure':
                 for n in node:
                     if n.tagname == 'caption':
-                        sectname = n.astext()
+                        sectname = clean_astext(n)
                         break
                 else:
                     continue
@@ -936,7 +930,7 @@ class BuildEnvironment:
         """Return a TOC nodetree -- for use on the same page only!"""
         toc = self.tocs[docname].deepcopy()
         for node in toc.traverse(nodes.reference):
-            node['refuri'] = node['anchorname']
+            node['refuri'] = node['anchorname'] or '#'
         return toc
 
     def get_toctree_for(self, docname, builder, collapse):
@@ -1080,7 +1074,7 @@ class BuildEnvironment:
                         # toctree originates
                         ref = toctreenode['parent']
                         if not title:
-                            title = self.titles[ref].astext()
+                            title = clean_astext(self.titles[ref])
                         reference = nodes.reference('', '',
                                                     refuri=ref,
                                                     anchorname='',
@@ -1183,7 +1177,7 @@ class BuildEnvironment:
                         docname, labelid = self.anonlabels.get(target, ('',''))
                         sectname = node.astext()
                         if not docname:
-                            self.warn(fromdocname, 'undefined label: %s' %
+                            self.warn(node['refdoc'], 'undefined label: %s' %
                                       target, node.line)
                     else:
                         # reference to the named label; the final node will
@@ -1192,7 +1186,7 @@ class BuildEnvironment:
                                                                      ('','',''))
                         if not docname:
                             self.warn(
-                                fromdocname,
+                                node['refdoc'],
                                 'undefined label: %s' % target + ' -- if you '
                                 'don\'t give a link caption the label must '
                                 'precede a section header.', node.line)
@@ -1218,17 +1212,17 @@ class BuildEnvironment:
                 elif typ == 'doc':
                     # directly reference to document by source name;
                     # can be absolute or relative
-                    docname = docname_join(fromdocname, target)
+                    docname = docname_join(node['refdoc'], target)
                     if docname not in self.all_docs:
-                        self.warn(fromdocname, 'unknown document: %s' % docname,
-                                  node.line)
+                        self.warn(node['refdoc'],
+                                  'unknown document: %s' % docname, node.line)
                         newnode = contnode
                     else:
                         if node['refcaption']:
                             # reference with explicit title
                             caption = node.astext()
                         else:
-                            caption = self.titles[docname].astext()
+                            caption = clean_astext(self.titles[docname])
                         innernode = nodes.emphasis(caption, caption)
                         newnode = nodes.reference('', '')
                         newnode['refuri'] = builder.get_relative_uri(
@@ -1238,7 +1232,8 @@ class BuildEnvironment:
                     # keywords are referenced by named labels
                     docname, labelid, _ = self.labels.get(target, ('','',''))
                     if not docname:
-                        #self.warn(fromdocname, 'unknown keyword: %s' % target)
+                        #self.warn(node['refdoc'],
+                        #          'unknown keyword: %s' % target)
                         newnode = contnode
                     else:
                         newnode = nodes.reference('', '')
@@ -1267,11 +1262,11 @@ class BuildEnvironment:
                                                            ('', ''))
                     if not docname:
                         if typ == 'term':
-                            self.warn(fromdocname,
+                            self.warn(node['refdoc'],
                                       'term not in glossary: %s' % target,
                                       node.line)
                         elif typ == 'citation':
-                            self.warn(fromdocname,
+                            self.warn(node['refdoc'],
                                       'citation not found: %s' % target,
                                       node.line)
                         newnode = contnode
@@ -1419,35 +1414,35 @@ class BuildEnvironment:
 
         for fn, entries in self.indexentries.iteritems():
             # new entry types must be listed in directives/other.py!
-            for type, string, tid, alias in entries:
+            for type, value, tid, alias in entries:
                 if type == 'single':
                     try:
-                        entry, subentry = string.split(';', 1)
+                        entry, subentry = value.split(';', 1)
                     except ValueError:
-                        entry, subentry = string, ''
+                        entry, subentry = value, ''
                     if not entry:
-                        self.warn(fn, 'invalid index entry %r' % string)
+                        self.warn(fn, 'invalid index entry %r' % value)
                         continue
                     add_entry(entry.strip(), subentry.strip())
                 elif type == 'pair':
                     try:
                         first, second = map(lambda x: x.strip(),
-                                            string.split(';', 1))
+                                            value.split(';', 1))
                         if not first or not second:
                             raise ValueError
                     except ValueError:
-                        self.warn(fn, 'invalid pair index entry %r' % string)
+                        self.warn(fn, 'invalid pair index entry %r' % value)
                         continue
                     add_entry(first, second)
                     add_entry(second, first)
                 elif type == 'triple':
                     try:
                         first, second, third = map(lambda x: x.strip(),
-                                                   string.split(';', 2))
+                                                   value.split(';', 2))
                         if not first or not second or not third:
                             raise ValueError
                     except ValueError:
-                        self.warn(fn, 'invalid triple index entry %r' % string)
+                        self.warn(fn, 'invalid triple index entry %r' % value)
                         continue
                     add_entry(first, second+' '+third)
                     add_entry(second, third+', '+first)
@@ -1455,8 +1450,15 @@ class BuildEnvironment:
                 else:
                     self.warn(fn, 'unknown index entry type %r' % type)
 
+        # sort the index entries; put all symbols at the front, even those
+        # following the letters in ASCII, this is where the chr(127) comes from
+        def keyfunc(entry, lcletters=string.ascii_lowercase + '_'):
+            lckey = entry[0].lower()
+            if lckey[0:1] in lcletters:
+                return chr(127) + lckey
+            return lckey
         newlist = new.items()
-        newlist.sort(key=lambda t: t[0].lower())
+        newlist.sort(key=keyfunc)
 
         # fixup entries: transform
         #   func() (in module foo)
@@ -1470,7 +1472,7 @@ class BuildEnvironment:
         i = 0
         while i < len(newlist):
             key, (targets, subitems) = newlist[i]
-            # cannot move if it hassubitems; structure gets too complex
+            # cannot move if it has subitems; structure gets too complex
             if not subitems:
                 m = _fixre.match(key)
                 if m:
@@ -1488,12 +1490,12 @@ class BuildEnvironment:
             i += 1
 
         # group the entries by letter
-        def keyfunc((k, v), ltrs=uppercase+'_'):
-            # hack: mutate the subitems dicts to a list in the keyfunc
+        def keyfunc((k, v), letters=string.ascii_uppercase + '_'):
+            # hack: mutating the subitems dicts to a list in the keyfunc
             v[1] = sorted((si, se) for (si, (se, void)) in v[1].iteritems())
             # now calculate the key
             letter = k[0].upper()
-            if letter in ltrs:
+            if letter in letters:
                 return letter
             else:
                 # get all other symbols under one heading
