@@ -7,7 +7,7 @@
     the doctree, thus avoiding duplication between docstrings and documentation
     for those who like elaborate docstrings.
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -322,7 +322,9 @@ class Documenter(object):
                 obj = self.get_attr(obj, part)
             self.object = obj
             return True
-        except (SyntaxError, ImportError, AttributeError), err:
+        # this used to only catch SyntaxError, ImportError and AttributeError,
+        # but importing modules with side effects can raise all kinds of errors
+        except Exception, err:
             self.directive.warn(
                 'autodoc can\'t import/find %s %r, it reported error: '
                 '"%s", please check your spelling and sys.path' %
@@ -965,10 +967,10 @@ class MethodDocumenter(ClassLevelDocumenter):
     """
     objtype = 'method'
     member_order = 50
+    priority = 0
 
     @classmethod
     def can_document_member(cls, member, membername, isattr, parent):
-        # other attributes are recognized via the module analyzer
         return inspect.isroutine(member) and \
                not isinstance(parent, ModuleDocumenter)
 
@@ -1011,11 +1013,18 @@ class AttributeDocumenter(ClassLevelDocumenter):
     objtype = 'attribute'
     member_order = 60
 
+    # must be higher than the MethodDocumenter, else it will recognize
+    # some non-data descriptors as methods
+    priority = 10
+
+    method_types = (FunctionType, BuiltinFunctionType, MethodType)
+
     @classmethod
     def can_document_member(cls, member, membername, isattr, parent):
-        return (isdescriptor(member) and not
-                isinstance(member, (FunctionType, BuiltinFunctionType))) \
-               or (not isinstance(parent, ModuleDocumenter) and isattr)
+        isdatadesc = isdescriptor(member) and not \
+                     isinstance(member, cls.method_types)
+        return isdatadesc or \
+               (isattr and not isinstance(parent, ModuleDocumenter))
 
     def document_members(self, all_members=False):
         pass
