@@ -5,17 +5,22 @@
 
     Highlight code blocks using Pygments.
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import sys
 import cgi
 import re
-import parser
 import textwrap
 
-from sphinx.util.texescape import tex_hl_escape_map
+try:
+    import parser
+except ImportError:
+    # parser is not available on Jython
+    parser = None
+
+from sphinx.util.texescape import tex_hl_escape_map_old, tex_hl_escape_map_new
 
 try:
     import pygments
@@ -122,7 +127,7 @@ class PygmentsBridge(object):
             # first, escape highlighting characters like Pygments does
             source = source.translate(escape_hl_chars)
             # then, escape all characters nonrepresentable in LaTeX
-            source = source.translate(tex_hl_escape_map)
+            source = source.translate(tex_hl_escape_map_old)
             return '\\begin{Verbatim}[commandchars=@\\[\\]]\n' + \
                    source + '\\end{Verbatim}\n'
 
@@ -155,6 +160,9 @@ class PygmentsBridge(object):
             # just replace all non-ASCII characters.
             src = src.encode('ascii', 'replace')
 
+        if parser is None:
+            return True
+
         try:
             parser.suite(src)
         except parsing_exceptions:
@@ -196,7 +204,10 @@ class PygmentsBridge(object):
                 return highlight(source, lexer, self.fmter[bool(linenos)])
             else:
                 hlsource = highlight(source, lexer, self.fmter[bool(linenos)])
-                return hlsource.translate(tex_hl_escape_map)
+                if hlsource.startswith(r'\begin{Verbatim}[commandchars=\\\{\}'):
+                    # Pygments >= 1.2
+                    return hlsource.translate(tex_hl_escape_map_new)
+                return hlsource.translate(tex_hl_escape_map_old)
         except ErrorToken:
             # this is most probably not the selected language,
             # so let it pass unhighlighted
